/*
 ****************************************************************************
 * BatchABEx
 *
 * Batch Background Extraction Dialog
 *
 * Input:   fit-files
 *
 * Output:  background subtracted image files
 *
 *          This dialog uses the AutomaticBackgroundExtractor
 *          with default settings
 *
 *
 * Copyright (C) 2014, Hartmut V. Bornemann
 *
 * Version history
 *          1.0    2014-05-24 first release
 ****************************************************************************
 */
#feature-id Batch Processing > BatchABEx
#include <pjsr/Color.jsh>
#include <pjsr/FrameStyle.jsh>
#include <pjsr/TextAlign.jsh>
#include <pjsr/StdButton.jsh>
#define StdCursor_ArrowQuestion 17
#define TITLE "BatchABEx"
#define VERSION "1.0.0"
#feature-info Batch Background Extraction.<br>\
   <br>\Written by Hartmut V. Bornemann.<br>\
   <br>\
   References: None<br>\
   <br>\
#define strict 0
//////////////////////////////////////////////////////////////////////////////
//
// Automatic Batch Background Extraction Dialog
//
//////////////////////////////////////////////////////////////////////////////


function ABEDialog()
{
   // Add all properties and methods of the core Dialog object to this object.
   this.__base__ = Dialog;
   this.__base__();

   var dlg = this;

   var column = 0;
   var maxx;
   var h = 24;
   var defaultProcess = "<default>";
   var processIcon = defaultProcess;

   var tbx = new TreeBox(this);
   with (tbx)
   {
      //
      // create a treebox with 2 columns
      // column 0 contains filenames
      // column 1 contains the full path
      //
      numberOfColumns = 2;
      position = new Point(8, 8);
      rootDecoration = false;
      height = 240;
      width  = 400;
      multipleSelection = true;
      setHeaderText(0, "Files");
      setHeaderText(1, "Full Path and FileNames");
      hideColumn(1);
      onNodeClicked = function( item, columnIndex )
      {
         enableButtons();
      }
   }

   var x = tbx.frameRect.right + 8;
   var y = tbx.frameRect.top;

   var btnAdd = new PushButton(this);
   with (btnAdd)
   {
      text = "Add Files";
      position = new Point(x, y);
      resize( 150 , height);
      y   += height + 8;
      maxx = frameRect.right;
      h    = height - 4;
      onClick  = function()
      {
         var ofd = new OpenFileDialog();
         ofd.multipleSelections = true;
         ofd.filters = [ [ "FITS oder PI Files", ".fit", ".fits", ".fts", ".XISF", "xisf" ] ];  // Ergänzung auf XISF Jörg Schneider

         if (ofd.execute())
         {
            for (var i = 0; i < ofd.fileNames.length; i++)
            {
               var filename =  ofd.fileNames[i];
               //
               // find filename in treebox column 1
               //
               var index = -1;
               for (var i = 0; i < tbx.numberOfChildren; i++)
               {
                  if (tbx.child(i).text(1) == filename)
                  {
                     index = i;
                     break;
                  }
               }
               if (index < 0)
               {
                  var n = new TreeBoxNode(tbx);
                  n.setText(0, File.extractNameAndExtension(filename));
                  n.setText(1, filename);
               }
            }
            enableButtons();
            lblStatus.text = tbx.numberOfChildren.toString();
         }
      }
   }

   var btnSelAll = new PushButton(this);
   with (btnSelAll)
   {
      enabled = false;
      text = "Select All";
      position = new Point(x, y);
      resize( 150 , height);
      y += height + 8;
      onClick  = function()
      {
         tbx.selectAllNodes();
         enableButtons();
      }
   }

   var btnInvertSel = new PushButton(this);
   with (btnInvertSel)
   {
      enabled = false;
      text = "Invert Selection";
      position = new Point(x, y);
      resize( 150 , height);
      y += height + 8;
      onClick  = function()
      {
         for (var i = 0; i < tbx.numberOfChildren; i++)
         {
            var n = tbx.child(i);
            n.selected = !n.selected;
         }
         enableButtons();
      }
   }

   var btnRemove = new PushButton(this);
   with (btnRemove)
   {
      enabled = false;
      text = "Remove Selected";
      position = new Point(x, y);
      resize( 150 , height);
      y += height + 8;
      onClick  = function()
      {
         var nodes  = tbx.selectedNodes;
         for (var i = 0; i < nodes.length; i++)
         {
            var c = tbx.childIndex(nodes[i]);
            tbx.remove(c);
         }
         enableButtons();
      }
   }

   var btnClear = new PushButton(this);
   with (btnClear)
   {
      enabled = false;
      text = "Clear";
      position = new Point(x, y);
      resize( 150 , height);
      y += height + 8;
      onClick  = function()
      {
         tbx.clear();
         enableButtons();
      }
   }

   var cbFullNames = new CheckBox(this);
   with (cbFullNames)
   {
      enabled = false;
      y = tbx.frameRect.bottom - height;
      position = new Point(x, y);
      text = "Full names";
      onCheck = function(checked)
      {
         tbx.hideColumn(column);
         column = 1-column;
         tbx.showColumn(column);
      }
   }

   //
   // Output file section
   //
   x  = tbx.frameRect.left;
   y  = tbx.frameRect.bottom + 16;

   var lblOutputDir = new Label(this);
   with (lblOutputDir)
   {
      text = "Output directory:";
      resize( 150 , height);
      position = new Point(x, y);
      textAlignment = TextAlign_Right;
   }

   var btnSelectDir = new Label(this);
   with (btnSelectDir)
   {
      lineWidth = 1;
      frameStyle =  FrameStyle_Box;
      resize(h , h);
      position = new Point(maxx - h, y);
      text = "...";
      onMousePress = function(x, y, button, buttonState, modifiers )
      {
         var dd = new GetDirectoryDialog();
         if (dd.execute())
         {
            tbOutputDir.text = dd.directory;
         }
      }
   }

   var tbOutputDir = new Edit(this);
   with (tbOutputDir)
   {
      position = new Point(lblOutputDir.frameRect.right + 8, y);
      resize( btnSelectDir.frameRect.left - position.x - 8 , h);
   }

   y  = lblOutputDir.frameRect.bottom + 8;

   var lblOutExtension = new Label(this);
   with (lblOutExtension)
   {
      text = "Output extension:";
      resize( 150 , height);
      position = new Point(x, y);
      textAlignment = TextAlign_Right;
   }

   var tbOutputExt = new Edit(this);
   with (tbOutputExt)
   {
      position = new Point(lblOutExtension.frameRect.right + 8, y);
      resize(tbOutputDir.width / 5, h);
      x = frameRect.right;
      text = ".fit";
   }

   var lblPrefix = new Label(this);
   with (lblPrefix)
   {
      text = "Prefix:  ";
      textAlignment = TextAlign_Right;
      resize( tbOutputDir.width / 5, height);
      position = new Point(x, y);
      x = frameRect.right;
   }

   var tbPrefix = new Edit(this);
   with (tbPrefix)
   {
      position = new Point(x, y);
      resize(tbOutputDir.width / 5, h);
      x = frameRect.right;
   }

   var lblPostfix = new Label(this);
   with (lblPostfix)
   {
      text = "Postfix:  ";
      textAlignment = TextAlign_Right;
      resize( tbOutputDir.width / 5, height);
      position = new Point(x, y);
      x = frameRect.right;
   }

   var tbPostfix = new Edit(this);
   with (tbPostfix)
   {
      position = new Point(x, y);
      resize(tbOutputDir.width / 5, h);
      x = frameRect.right;
      text = "_b";
   }

   y = tbOutputExt.frameRect.bottom + 16;
   x = tbOutputExt.frameRect.left;

   var cbOverwrite = new CheckBox(this);
   with (cbOverwrite)
   {
      position = new Point(x, y);
      text = "Overwrite existing files";
      y += height;
   }

   x  = tbx.frameRect.left;
   y  = cbOverwrite.frameRect.bottom + 8;

   var lblProcessIcon = new Label(this);
   with (lblProcessIcon)
   {
      text = "Process Icon:";
      resize( 150 , height);
      position = new Point(x, y);
      textAlignment = TextAlign_Right;
      x += width + 8;
   }

   var IconIdsComboBox = new ComboBox( this );
   with (IconIdsComboBox)
   {
      resize( 150 , height);
      position = new Point(x, y);
      addItem(defaultProcess);
      var icons = ProcessInstance.iconsByProcessId("AutomaticBackgroundExtractor");
      for (var i = 0; i < icons.length; i++) addItem(icons[i]);
      y += height;
      onItemSelected = function( item )
      {
         processIcon = this.itemText( item );
      }
   }

   y += 16;
   x  = tbOutputExt.frameRect.left;

   var btnExec = new PushButton(this);
   with (btnExec)
   {
      text = "Execute";
      position = new Point(x, y);
      onClick = function()
      {
         btnReady.enabled = false;
         for (var i = 0; i < tbx.numberOfChildren; i++)
         {
            if (Console.abortRequested)
            {
               Console.writeln("Abort requested");
               break;
            }
            var input_filename = tbx.child(i).text(1);
            //
            // create output filename
            //
            var output_filename;
            if (tbOutputDir.text != "")
            {
               if (File.directoryExists(tbOutputDir.text))
                  output_filename = tbOutputDir.text;
            }
            if (output_filename == null)
            {
               output_filename  = File.extractDrive(input_filename);
               output_filename += File.extractDirectory(input_filename);
            }
            output_filename += '\/';
            output_filename += tbPrefix.text;
            output_filename += File.extractName(input_filename);
            output_filename += tbPostfix.text;
            output_filename += tbOutputExt.text;

            lblStatus.text = (i+1).toString()+ " (" + tbx.numberOfChildren.toString() + ")";
            lblStatus.update();
            run(processIcon, input_filename, output_filename, cbOverwrite.checked);
         }
         btnReady.enabled = true;
      }
   }

   var lblStatus = new Label(this);
   with (lblStatus)
   {
      backgroundColor = Color.rgbaColor(255, 255, 255, 255);
      textAlignment = TextAlign_Center;
      lineWidth = 1;
      resize (80, h);
      position = new Point(btnExec.frameRect.right + 8, y);
   }

   var btnReady = new PushButton(this);
   with (btnReady)
   {
      text = "Close";
      position = new Point(maxx - width, y);
      onClick = function()
      {
         dlg.ok();
      }
   }

   // my ©

   var lblCopyright = new Label(this)
   with (lblCopyright)
   {
      position  = new Point(8, y);
      text      = "©";
      cursor    = new Cursor(StdCursor_ArrowQuestion);
      onMousePress = function(x, y, button, buttonState, modifiers )
      {
        var dlg = new helpdlg();
        dlg.execute();
      }
   }

   with (this)
   {
      width  = maxx + 8;
      height = btnReady.frameRect.bottom + 8;
      windowTitle = "Automatic Batch Background Extraction";
      userResizable = false;
   }

   function enableButtons()
   {
      btnSelAll.enabled = tbx.numberOfChildren > 0;
      btnClear.enabled  = tbx.numberOfChildren > 0;
      btnRemove.enabled = tbx.selectedNodes.length > 0;
      btnInvertSel.enabled = tbx.selectedNodes.length > 0;
      cbFullNames.enabled  = tbx.numberOfChildren > 0;
   }

   function run(processIcon, input_filename, output_filename, overwrite)
   {
      if (!File.exists(input_filename))
      {
         Console.writeln("File does not exist: " + input_filename);
         return;
      }
      if (File.exists(output_filename) && !overwrite)
      {
         Console.writeln("File already exists: " + output_filename);
         return;
      }

      ImageWindow.open( input_filename );
      var w = ImageWindow.openWindows[ImageWindow.openWindows.length-1];

      if (!w || w.isNull)
      {
         Console.writeln("Error opening file: " + input_filename);
         return;
      }

      Console.writeln("Processing  file:   " + input_filename);

      var process;
      if (processIcon == defaultProcess)
      {
         process = new AutomaticBackgroundExtractor;
         with (process)
         {
            tolerance = 1.000;
            deviation = 0.800;
            unbalance = 1.800;
            minBoxFraction = 0.050;
            maxBackground = 1.0000;
            minBackground = 0.0000;
            useBrightnessLimits = false;
            polyDegree = 5;
            boxSize = 5;
            boxSeparation = 5;
            modelImageSampleFormat = AutomaticBackgroundExtractor.prototype.f32;
            abeDownsample = 2.00;
            writeSampleBoxes = false;
            justTrySamples = false;
            targetCorrection = AutomaticBackgroundExtractor.prototype.Subtract;
            normalize = true;
            discardModel = true;
            replaceTarget = true;
            correctedImageId = "";
            correctedImageSampleFormat = AutomaticBackgroundExtractor.prototype.SameAsTarget;
            verboseCoefficients = false;
            compareModel = false;
            compareFactor = 10.00;
         }
      }
      else
      {
         process = ProcessInstance.fromIcon(processIcon);
         with (process)
         {
            writeSampleBoxes = false;
            justTrySamples = false;
            replaceTarget = true;
            correctedImageSampleFormat = AutomaticBackgroundExtractor.prototype.SameAsTarget;
         }
      }

      with (process)
      {
         executeOn(w.mainView);
         //
         // save
         //
         w.saveAs(output_filename, false, false, true, !overwrite);
      }
      w.close();
   }
}

ABEDialog.prototype = new Dialog;


function helpdlg()
{
   this.__base__ = Dialog;
   this.__base__();

   this.minHeight = 400;
   this.minWidth  = 500;

   this.windowTitle = "BatchABEx Help";

   this.OK_Button = new PushButton (this);
   with (this.OK_Button)
   {
      text = "OK";
      maxWidth = 100;

      onClick = function()
      {
         this.parent.ok();
      }
   }

   this.helpText = new TextBox( this );
   with ( this.helpText )
   {
      frameStyle = FrameStyle_Box;
      margin = 4;
      readOnly = true;
      wordWrapping = true;
      useRichText = true;

      text = "<b>" + TITLE + " V" + VERSION +
      " Batch Background Extraction Dialog.</b>" +
      "<p><b>" + TITLE + "</b> Equalizes a stack of selected images using the " +
      "<b>AutomaticBackgroundExtractor</b> process</p>" +
      "<p><b>Purpose: </b> Flatten background of images prior to stacking." +
      " This can be useful, if the image gradient changes with each frame. </p>" +
      "<p><b>Usage: </b> Press Add-button and select images from the file dialog for this process." +
      " The new files will be stored in the same directory or in a new output directory," +
      " if choosen. Change also output file name options, if neccessary." +
      " Individual settings of <b>Sample Generation and Rejection</b> or" +
      " <b>Interpolation and Output</b> options can be made by creating a process icon" +
      " from AutomaticBackgroundExtractor and selecting the icon from the combobox.</p>" +
      "<p>Press <b>Execute</b> and wait until ready." +
      " Early execution termination is possible by clicking Pause/Abort in the console window.</p>" +
      "<p>Copyright © Hartmut Bornemann, 2014, mailto:hvb356@yahoo.com</p>";
   }

   this.buttonSizer = new HorizontalSizer;
   with ( this.buttonSizer )
   {
      spacing = 4;
      add (this.OK_Button);
   }


   this.sizer = new VerticalSizer;
   with ( this.sizer )
   {
      margin = 6;
      spacing = 6;
      add( this.helpText );
      add( this.buttonSizer );
   }

   this.helpText.home();
}

helpdlg.prototype = new Dialog;

function main()
{
   Console.abortEnabled = true;
   Console.show();
   var dialog = new ABEDialog();
   dialog.execute();
}

main();

// ****************************************************************************
// EOF BatchABEx.js - Released 2014/05/22 12:00:00 UTC
